/*
 * File:  def.c
 * Date:  September 14, 1993
 * Programmer:  Cecil Huang
 * =========================
 * This file implements the basic functions used in the source code
 * for SWIG1.12.
 */


#include "def.h"
#include "io.h"
#include <stdio.h>
#include <stdlib.h>


PROTEIN *make_protein ()
{
  return (PROTEIN *) malloc (sizeof (PROTEIN));
}


void free_protein (PROTEIN *protein)
{
  free (protein->sequence);
  free (protein);
}


CELL **make_CELL_table (int rows, int columns)
{
  int i;
  CELL **mat;
  
  mat = (CELL **) malloc (rows * sizeof (CELL *));

  for (i = 0; i < rows; i++) {
    mat[i] = (CELL *) calloc (columns, sizeof(CELL));
  }

  return (mat);
}


void free_CELL_table (CELL **mat, int rows)
{
  int i;

  for (i = 0; i < rows; i++) {
    free (mat[i]);
  }

  free (mat);
}


MAT_PLUS *make_MAT_PLUS(struct protein *p_database,
			struct protein *p_query)
{
  int columns, rows;
  MAT_PLUS *mp;

  rows               = p_query->length + 1;
  columns            = p_database->length + 1;

  mp                 = (MAT_PLUS *) malloc(sizeof(MAT_PLUS));
  mp->mat            = make_CELL_table(2, columns);
  mp->mat_align      = make_int_table(rows, columns);
  mp->mat_align_rows = rows;
  mp->max_value      = 0.0;
  mp->rmax           = rows;
  mp->cmax           = columns;
  mp->p_query        = p_query;
  mp->p_database     = p_database;

  return (mp);
}


void free_MAT_PLUS (MAT_PLUS *mp)
{
  free_int_table (mp->mat_align, mp->mat_align_rows);
  free_CELL_table (mp->mat, 2);

  free (mp);
}


SSTRING *make_sstring (int len)
{
  SSTRING *k;

  k = (SSTRING *) malloc (sizeof (SSTRING));
  k->string = (char *) malloc (sizeof (char) * (len + 1));
  k->pos = len;
  k->string[len] = '\0';

  return (k);
}


void free_sstring (SSTRING *k)
{
  free (k->string);
  free (k);
}


void ss_append (char c, SSTRING *ss)
{
  if (ss->pos == 0) {
     printf ("String overflow.\n");
     fclose (stdout);
     exit(-1);
   }

  ss->string[--ss->pos] = c;
}


char *ss_string (SSTRING *ss)
{
  return (ss->string + ss->pos);
}


int *make_int_array (int len)
{
  return (int *) malloc (len * sizeof (int));
}


int *extend_int_array (int *array, int newlen)
{
  return (int *) realloc (array, newlen * sizeof (int));
}


char *make_char_array (int len)
{
  return (char *) malloc (len * sizeof (char));
}


char *extend_char_array (char *array, int newlen)
{
  return (char *) realloc (array, newlen * sizeof (char));
}


/* NOTE:  blank line has length of 1.  Why?  \n.  */

int getline (FILE *fp, char *line, int max)
{
  if (fgets(line, max, fp) == NULL) {
    return 0;
  } else {
    return strlen (line);
  }
}


int gap_penalty_interp_index (char *s)
{
  if (!strcmp (s, "ABSOLUTE")) {
    return (ABSOLUTE);
  }

  if (!strcmp (s, "NORMALIZED-WEIGHTED")) {
    return (NORM_WEIGHTED);
  }

  if (!strcmp (s, "NORMALIZED-UNWEIGHTED")) {
    return (NORM_UNWEIGHTED);
  }

  return (0);
}
