/*
 * File:  params.h
 * Date:  September 14, 1993
 * Programmer:  Cecil Huang
 * =========================
 * This code parses the parameter input file and does some
 * preliminary checks on its contents.  Warning -- the code
 * is somewhat ugly, and a lot of stuff is hardwired in.
 * didn't have time to come up with an elegant, PDA-type
 * parser like you see in real compilers.  But for this
 * assignment it's probably not worth the effort.
 *
 * In any case, this code is guaranteed to be robust provided
 * that no modifications are made to it.  At least be glad that
 * all the ugliness is modularized away into this file!
 *
 * Updated September 14, 1993 by Cecil Huang.  Ignores unrecognized
 * params instead of crashing.
 */


#include <ctype.h>
#include "def.h"
#include "params.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


extern char
  algo[MAXLINE], database_pepfile[MAXLINE], query_pepfile[MAXLINE],
  gap_penalty_interp[MAXLINE], matrix_file[MAXLINE],
  dgap_vary[MAXLINE], dgap_file[MAXLINE],
  qgap_vary[MAXLINE], qgap_file[MAXLINE],
  outfile[MAXLINE], matfile[MAXLINE];

static char scratch[MAXLINE];


/* This is sensitive to order in def.h.  DO NOT CHANGE! */

static char
  *var_value_loc[NUMVARS] = {
    algo, database_pepfile, query_pepfile, gap_penalty_interp, matrix_file,
    dgap_vary, dgap_file,
    qgap_vary, qgap_file,
    outfile, matfile};

static char
  *valid_vars[NUMVARS] = {
    "algorithm",
    "database_file",
    "query_file",
    "gap_penalty",
    "matrix_file",
    "database_gap_option",
    "database_gap_file",
    "query_gap_option",
    "query_gap_file",
    "output_alignment_file",
    "edit_matrix_file"
    };

static int line_num, err_num;


/*
 * I thought about implementing a hash table, but I said forget it.
 */

void error_msg (void)
{
  fprintf (stderr, "Line %d: ", line_num);
  fclose (stderr);
  err_num++;
}


void set_param_defaults (void)
{
}


void myabort (void)
{
  fprintf (stdout, "aborting ...\n");
  fclose (stdout);

  exit (-1);
}


int get_var_index (char *x)
{
  int i;


  for (i = 0; i < 11; i++) {
    if (!strncmp (x, valid_vars[i], MAXSTRING)) {
      return (i);
    }
  }

  return (-1);
}


void get_input_params (char *filename)
{
  FILE *fp;
  char parse_variable[MAXSTRING], parse_value[MAXLINE];
  int num, var_index;


  set_param_defaults ();
  fp = open_file_read (filename);

  err_num = 0;

  for (line_num = 1; getline(fp, scratch, MAXLINE); line_num++) {
    if (scratch[0] == '#') {
      continue;
    }

    num = sscanf (scratch, "%s = %s", parse_variable, parse_value);

    if (num < 1) {
      continue;
    }

    if (num != 2) {
      error_msg ();
      fprintf (stderr, "syntax error near string \"%s\"\n", parse_variable);
      continue;
    }

    var_index = get_var_index (parse_variable);

    if (var_index == -1) {
      error_msg ();
      fprintf (stderr, "unrecognized parameter \"%s\"\n", parse_variable);
      fprintf (stderr, "**** Parameter ignored.\n");
      err_num--;

      continue;
    }

    strcpy (var_value_loc[var_index], parse_value);
  }

  if (err_num > 0) {
    fprintf (stderr, "**** Total %d errors.  ", err_num);
    fclose (stderr);
    myabort ();
  }

  fclose (fp);
}


int isident (char c)
{
  return (isalnum ((int) c) || (c == '_'));
}


FILE *open_file_read (char *filename)
{
  FILE *fp;


  fp = fopen (filename, "r");

  if (fp == NULL) {
    fprintf (stderr, "Error:  file \"%s\" not found\n", filename);
    fclose (stderr);
    myabort ();
  }

  return (fp);
}


void print_params (void)
{
  int i;


  for (i = 0; i < 11; i++) {
    fprintf (stdout, "%-30s = %s\n", valid_vars[i], var_value_loc[i]);
  }

  fclose (stdout);
}


void check_file_read_error (char *filename)
{
  if (access (filename, R_OK)) {
    fprintf (stderr, 
	     "Error:  file \"%s\" not found or could not be accessed\n",
	     filename);
    fclose (stderr);
    err_num++;
  }
}


void check_file_write_error (char *filename)
{
  FILE *fp;


  if ((fp = fopen(filename, "w")) != NULL) {
    fclose (fp);
    sprintf (scratch, "rm -f \"%s\"", filename);
    system (scratch);
  } else {
    fprintf (stderr, "Error:  cannot write to file \"%s\":  check access\n",
	     filename);
    err_num++;
  }

  fclose (stderr);
}


int algo_index (char *s)
{
  if (!strcmp(s, "SMITH-WATERMAN")) {
    return (SMITH_WATERMAN);
  }

  if (!strcmp(s, "NEEDLEMAN-WUNSCH")) {
    return (NEEDLEMAN_WUNSCH);
  }

  if (!strcmp(s, "PROFILE")) {
    return (PROFILE);
  }

  return (0);
}


int gap_vary_index (char *s)
{
  if (!strcmp (s, "CONSTANT")) {
    return (CONSTANT);
  }

  if (!strcmp (s, "VARIABLE")) {
    return (VARIABLE);
  }

  return (0);
}


void check_params (void)
{
  int i, c;


  err_num = 0;

  for (i = 0; i < 9; i++) {
    if (var_value_loc[i][0] == '\0') {
      fprintf (stderr, "Error:  variable %s unspecified\n", valid_vars[i]);
      err_num++;
    }
  }

  if (err_num > 0) {
    fprintf (stderr, "**** Total %d errors.  ", err_num);
    fclose (stderr);
    myabort ();
  }

  err_num = 0;

  if (!(c = algo_index (algo))) {
    fprintf (stderr, "Error:  unrecognized algorithm => %s\n", algo);
    err_num++;
  }
  if (c == PROFILE) {
    fprintf (stderr, "Error:  PROFILE algorithm not implemented.\n");
    err_num++;
  }

  check_file_read_error (database_pepfile);
  check_file_read_error (query_pepfile);

  if (!gap_penalty_interp_index (gap_penalty_interp)) {
    fprintf (stderr, "Error:  unrecognized gap_penalty => %s\n",
	     gap_penalty_interp);
    err_num++;
  }

  check_file_read_error (matrix_file);

  if (!(c = gap_vary_index (qgap_vary))) {
    fprintf (stderr, "Error:  unrecognized query_gap_option => %s\n",
	     qgap_vary);
    err_num++;
  }

  if (c == VARIABLE) {
    fprintf (stderr, "Error:  VARIABLE query gap option unimplemented.\n");
    err_num++;
  }

  check_file_read_error (qgap_file);

  if (!(c = gap_vary_index (dgap_vary))) {
    fprintf (stderr, "Error:  unrecognized database_gap_option => %s\n",
	     dgap_vary);
    err_num++;
  }

  if (c == VARIABLE) {
    fprintf 
      (stderr, "Error:  VARIABLE database gap option unimplemented.\n");
    err_num++;
  }

  check_file_read_error (dgap_file);

  if (outfile[0] != '\0') {
    check_file_write_error (outfile);
  }

  if (matfile[0] != '\0') {
    check_file_write_error (matfile);
  }

  if (err_num > 0) {
    fprintf (stderr, "**** Total %d errors.  ", err_num);
    fclose (stderr);
    myabort ();
  }
}


void parse_and_check_params (char *filename)
{
  get_input_params (filename);
  check_params ();
}
